(*
 Oukseh Lee

 Copyright(c) 2000-2004 KAIST/SNU Research On Program Analysis System
 (National Creative Research Initiative Center 1998-2003)
 http://ropas.snu.ac.kr/n

 All rights reserved. This file is distributed under the terms of
 an Open Source License.
*)

structure EnvPrint = struct
  open Format
  open Ty
  open NEnv

  fun separate() = printf "@\n"
  fun none _ _ = ()

  fun tyvarl n = if n=0 then () else if n=1 then printf "'a " else
    let fun f i = if i=n then []
                  else let val s = Tv.make_tvname i
                       in  s :: f (i+1)
                       end
    in printf "(@[";
       TyPrint.pp_list (printf "'%s") (fn _ => printf ",@ ") (f 0);
       printf ") "
    end

  fun print_map sw m g h f = (* switch map seperator iterator element-function *)
        h (fn x y => f x y (fn x=>if !sw then g() else sw := true)) m
  fun print_list sw m g f = (* switch map seperator element-function *)
        List.iter (fn x => f x (fn x=>if !sw then g() else sw := true)) m

  fun tystr ts c id = case ts of
        TS.TsName x =>
          if Tn.context x <> c || Tn.name x <> id then 
            (printf "@ =@ "; tyvarl (Tn.arity x); TyPrint.tynamec x c)
          else 
            let val cl = CE.find x
            in  case cl of [] => () 
                         | _ => let val sw = ref false
                                in  printf "@ =@ @[<-2>";
                                    print_list sw cl (fn _ => printf "@;| ")
                                      (fn (i,None) g => g(); printf "%s" i
                                        | (i,Some t) g => g(); printf "@[<2>%s of@ " i;
                                                          TyPrint.typc t c;
                                                          printf "@]");
                                    printf "@]"
                                end
            end
      | TS.TsFun(n,t) => printf "@ =@ @["; TyPrint.typc t c; printf "@]"

  fun printable_ve x = x < "A" || x > "Z"
  fun printable_te x = true
  fun printable_se x = true
  fun count_printable (Env(s,t,v)) =
    let val counter = ref 0
    in  SE.iter (fn k _ => if printable_se k then counter++) s;
        TE.iter (fn k _ => if printable_te k then counter++) t;
        VE.iter (fn k _ => if printable_ve k then counter++) v;
        !counter
    end

  fun transty t =
    case t of
      Ty.Link t => transty !t
    | Ty.Tuple l => Nty.NtTuple (List.map transty l)
    | Ty.Data(l,n) => Nty.NtData (List.map transty l,n)
    | Ty.Rec(m,_) => Nty.NtRec(List.map (fn (l,t) => (l, transty t)) m)
    | _ => Nty.NtSome

  fun anti_transty t =
    case t of
      Nty.NtTuple l => Ty.Tuple (List.map anti_transty l)
    | Nty.NtData(l, n) => Ty.Data(List.map anti_transty l, n)
    | Nty.NtRec m => Ty.Rec(List.map (fn (l,t) => (l, anti_transty t)) m, None)
    | _ => Ty.Var(Ty.Meta(0,0))

  fun exnargsty lid =
    let fun nlid' t = case t of
            Longident.Lident s => [(s, Location.none)]
          | Longident.Ldot(t,s) => nlid' t @ [(s, Location.none)]
          | _ => raise Not_found
        fun nlid t = case t of
            Longident.Lident s => ([], (s, Location.none), Location.none)
          | Longident.Ldot (t,s) => (nlid' t, (s, Location.none), Location.none)
          | _ => raise Not_found
        val (ts,_) = find_ty (nlid lid)
    in  case exn_arg (in_type ts) of
          Some t => transty t
        | None => raise Not_found
    end

  fun argsty tn tl c =
      let val (_, atopt) = List.find (fn (x,arg) => x = c) (CE.find tn)
      in  case atopt of
            None => []
          | Some t => 
             let val t' = Ty.copy Location.none (List.map anti_transty tl) t
             in  [transty t']
             end
      end
  fun excn f = f (transty Ty.type_exn, argsty, exnargsty)

  fun ve m c sw print_val = print_map sw m separate VE.iter 
    (fn id (n,t) g => if printable_ve id then 
                          (g();
                           printf "val %s@[: @[" id;
                           TyPrint.typc t c;
                           printf "@]";
                           print_val id  (transty t, argsty, exnargsty);
                           printf "@]")
                      else if is_exn t then 
                          (g();
                           printf "@[exception %s" id;
                           (case exn_arg t of
                              Some t => (printf " of @["; TyPrint.typc t c; printf "@]")
                            | _ => ());
                           printf "@]"))
                    

  fun te m c sw = print_map sw m separate TE.iter
    (fn id e g => if printable_te id then (
                    g();
		    printf "@[<5>type ";
		    (case e of
		       TS.TsFun (n,_) => tyvarl n
                     | TS.TsName tn => tyvarl (Tn.arity tn));
                    printf "%s" id;
                    tystr e c id;
                    printf "@]";
                    sw := true))

  fun e(env as Env(s,t,v)) c f = 
    let val sw = ref false
        val items = count_printable env
    in  printf "@[@[<2>sig";
        if items=1 then printf "@ " else if items>1 then printf "@\n";
        te t c sw;
        se s c sw;
        ve v c sw f;
        if items<=1 then printf "@]@ " else printf "@]@\n";
        printf "end@]"
    end

  and se m c sw = print_map sw m separate SE.iter
    (fn id env g => if printable_se id then ( 
           g();
           printf "@[<2>structure %s:@ " id;
           e env (id::c) none;
           printf "@]"))

  and print_g m sw = G.M.iter (fn k (t,e') => 
                       if !sw then printf "@\n" else sw := true;
                       printf "@[<2>signature %s =@ " k;
                       e e' (sig_context' k :: !base_context) none;
                       printf "@]") m

  fun print_f m sw = 
    let
      fun print_sig'(t,e') = case e' of Env(s,_,_) =>
            let val sw = ref false
            in  se s (functor_context()::!base_context) sw
            end
    in  
        F.M.iter (fn k (t,l,e1,(t',e2)) => 
                  (if !sw then printf "@\n" else sw := true;
                   printf "@[<2>functor %s@ (@[" k;
                   (case l of [sid] => (printf "%s:@ " sid; 
                                        e e1 (sid :: !base_context) none)
                            | _ => print_sig' (t,e1));
                   printf "@]) ->@ ";
                   e e2 (fct_context' k :: !base_context) none; 
                   printf "@]")) m
    end

  fun estack l g sw = 
    let val c = !base_context
        fun f l = case l of
            NEnv.RealEnv(Env(s,t,v))::tail => (
              te t c sw;
              se s c sw;
              ve v c sw g;
              f tail)
          | NEnv.OpenEnv(e',l)::t => f t
          | _ => ()
    in  f l
    end
   
  fun b (a:NEnv.b) h = 
        let val sw = ref false
        in  printf "@["; 
            print_g a.sigs sw; 
            print_f a.fcts sw; 
            estack (!(a.env)) h sw; 
            printf "@."
        end
end
